// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure.KeyVault.Internal;

namespace Microsoft.Azure.KeyVault.Internal
{
    /// <summary>
    /// Operations for secrets in a vault
    /// </summary>
    internal partial interface ISecretOperations
    {
        /// <summary>
        /// Delete the specified secret
        /// </summary>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a secret operation request.
        /// </returns>
        Task<SecretResponseMessageWithRawJsonContent> DeleteAsync(string secretIdentifier, CancellationToken cancellationToken);
        
        /// <summary>
        /// Gets a secret
        /// </summary>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a secret operation request.
        /// </returns>
        Task<SecretResponseMessageWithRawJsonContent> GetAsync(string secretIdentifier, CancellationToken cancellationToken);
        
        /// <summary>
        /// List the secrets in the specified vault
        /// </summary>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a secret operation request.
        /// </returns>
        Task<SecretResponseMessageWithRawJsonContent> ListAsync(string vault, int? top, CancellationToken cancellationToken);
        
        /// <summary>
        /// List the next page of secrets in the specified vault
        /// </summary>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a secret operation request.
        /// </returns>
        Task<SecretResponseMessageWithRawJsonContent> ListNextAsync(string nextLink, CancellationToken cancellationToken);
        
        /// <summary>
        /// List the versions of a secret in the specified vault
        /// </summary>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a secret operation request.
        /// </returns>
        Task<SecretResponseMessageWithRawJsonContent> ListVersionsAsync(string vault, string secretName, int? top, CancellationToken cancellationToken);
        
        /// <summary>
        /// List the versions of a secret in the specified vault
        /// </summary>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a secret operation request.
        /// </returns>
        Task<SecretResponseMessageWithRawJsonContent> ListVersionsNextAsync(string nextLink, CancellationToken cancellationToken);
        
        /// <summary>
        /// Sets a secret in the specified vault.
        /// </summary>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a secret operation request.
        /// </returns>
        Task<SecretResponseMessageWithRawJsonContent> SetAsync(string secretIdentifier, SecretRequestMessageWithRawJsonContent request, CancellationToken cancellationToken);
        
        /// <summary>
        /// Update the specified secret
        /// </summary>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a secret operation request.
        /// </returns>
        Task<SecretResponseMessageWithRawJsonContent> UpdateAsync(string secretIdentifier, SecretRequestMessageWithRawJsonContent request, CancellationToken cancellationToken);
    }
}
