// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using Microsoft.Azure.Management.DataFactories.Models;

namespace Microsoft.Azure.Management.DataFactories.Models
{
    /// <summary>
    /// The data source that could be accessed from on-premises machines using
    /// the specific ODBC driver.
    /// </summary>
    public partial class OnPremisesOdbcLinkedService : LinkedServiceProperties
    {
        private string _authenticationType;
        
        /// <summary>
        /// Required. Gets or sets the authentication type used to access the
        /// data source, e.g. windows authentication (Windows), database
        /// authentication (Database) or anonymous authentication (Anonymous).
        /// </summary>
        public string AuthenticationType
        {
            get { return this._authenticationType; }
            set { this._authenticationType = value; }
        }
        
        private string _credentialConnectionString;
        
        /// <summary>
        /// Optional. Gets or sets the access credential portion of the
        /// connection string specified in driver-specific property-value
        /// format, e.g. "Uid=<user ID>;Pwd=<password>;RefreshToken=<secret
        /// refresh token>;".
        /// </summary>
        public string CredentialConnectionString
        {
            get { return this._credentialConnectionString; }
            set { this._credentialConnectionString = value; }
        }
        
        private string _encryptedCredentialConnectionString;
        
        /// <summary>
        /// Optional. Gets or sets on-premises gateway generated connection
        /// string which contains the encrypted access credential.
        /// </summary>
        public string EncryptedCredentialConnectionString
        {
            get { return this._encryptedCredentialConnectionString; }
            set { this._encryptedCredentialConnectionString = value; }
        }
        
        private string _gatewayName;
        
        /// <summary>
        /// Required. Gets or sets the on-premises gateway name.
        /// </summary>
        public string GatewayName
        {
            get { return this._gatewayName; }
            set { this._gatewayName = value; }
        }
        
        private string _nonCredentialConnectionString;
        
        /// <summary>
        /// Optional. Gets or sets the non-credential portion of the connection
        /// string specified in driver-specific property-value format, e.g.
        /// "Driver=<ODBC driver name>;Server=<server name>;Database=<database
        /// name>;".
        /// </summary>
        public string NonCredentialConnectionString
        {
            get { return this._nonCredentialConnectionString; }
            set { this._nonCredentialConnectionString = value; }
        }
        
        private string _password;
        
        /// <summary>
        /// Optional. Gets or sets the on-premises windows authentication
        /// password.
        /// </summary>
        public string Password
        {
            get { return this._password; }
            set { this._password = value; }
        }
        
        private string _userName;
        
        /// <summary>
        /// Optional. Gets or sets the on-premises windows authentication user
        /// name.
        /// </summary>
        public string UserName
        {
            get { return this._userName; }
            set { this._userName = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the OnPremisesOdbcLinkedService class.
        /// </summary>
        public OnPremisesOdbcLinkedService()
        {
        }
        
        /// <summary>
        /// Initializes a new instance of the OnPremisesOdbcLinkedService class
        /// with required arguments.
        /// </summary>
        public OnPremisesOdbcLinkedService(string gatewayName, string authenticationType)
            : this()
        {
            if (gatewayName == null)
            {
                throw new ArgumentNullException("gatewayName");
            }
            if (authenticationType == null)
            {
                throw new ArgumentNullException("authenticationType");
            }
            this.GatewayName = gatewayName;
            this.AuthenticationType = authenticationType;
        }
    }
}
