// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Linq;
using Hyak.Common;
using Microsoft.WindowsAzure.Management.Compute.Models;

namespace Microsoft.WindowsAzure.Management.Compute.Models
{
    /// <summary>
    /// Objects that provide system or application data.
    /// </summary>
    public partial class ConfigurationSet
    {
        private string _adminPassword;
        
        /// <summary>
        /// Optional. Specifies the string representing the administrator
        /// password to use for the virtual machine. If the VM will be created
        /// from a 'Specialized' VM image, the password is not required.
        /// </summary>
        public string AdminPassword
        {
            get { return this._adminPassword; }
            set { this._adminPassword = value; }
        }
        
        private string _adminUserName;
        
        /// <summary>
        /// Optional. Specifies the name that is used to rename the default
        /// administrator account. If the VM will be created from a
        /// 'Specialized' VM image, the user name is not required.
        /// </summary>
        public string AdminUserName
        {
            get { return this._adminUserName; }
            set { this._adminUserName = value; }
        }
        
        private string _computerName;
        
        /// <summary>
        /// Optional. Specifies the computer name for the virtual machine. If
        /// the computer name is not specified, a name is created based on the
        /// name of the role. Computer names must be 1 to 15 characters in
        /// length. This element is only used with the
        /// WindowsProvisioningConfiguration set.
        /// </summary>
        public string ComputerName
        {
            get { return this._computerName; }
            set { this._computerName = value; }
        }
        
        private string _configurationSetType;
        
        /// <summary>
        /// Optional. Specifies the configuration type for the configuration
        /// set.
        /// </summary>
        public string ConfigurationSetType
        {
            get { return this._configurationSetType; }
            set { this._configurationSetType = value; }
        }
        
        private string _customData;
        
        /// <summary>
        /// Optional. Optional. Provides base64 encoded custom data to be
        /// passed to VM.
        /// </summary>
        public string CustomData
        {
            get { return this._customData; }
            set { this._customData = value; }
        }
        
        private bool? _disableSshPasswordAuthentication;
        
        /// <summary>
        /// Optional. Specifies whether or not SSH authentication is disabled
        /// for the password. This element is only used with the
        /// LinuxProvisioningConfiguration set. By default this value is set
        /// to true.
        /// </summary>
        public bool? DisableSshPasswordAuthentication
        {
            get { return this._disableSshPasswordAuthentication; }
            set { this._disableSshPasswordAuthentication = value; }
        }
        
        private DomainJoinSettings _domainJoin;
        
        /// <summary>
        /// Optional. Contains properties that specify a domain to which the
        /// virtual machine will be joined. This element is only used with the
        /// WindowsProvisioningConfiguration set.
        /// </summary>
        public DomainJoinSettings DomainJoin
        {
            get { return this._domainJoin; }
            set { this._domainJoin = value; }
        }
        
        private bool? _enableAutomaticUpdates;
        
        /// <summary>
        /// Optional. Specifies whether automatic updates are enabled for the
        /// virtual machine. This element is only used with the
        /// WindowsProvisioningConfiguration set. The default value is false.
        /// </summary>
        public bool? EnableAutomaticUpdates
        {
            get { return this._enableAutomaticUpdates; }
            set { this._enableAutomaticUpdates = value; }
        }
        
        private string _hostName;
        
        /// <summary>
        /// Optional. Specifies the host name for the VM. Host names are ASCII
        /// character strings 1 to 64 characters in length. This element is
        /// only used with the LinuxProvisioningConfiguration set.
        /// </summary>
        public string HostName
        {
            get { return this._hostName; }
            set { this._hostName = value; }
        }
        
        private IList<InputEndpoint> _inputEndpoints;
        
        /// <summary>
        /// Optional. Contains a collection of external endpoints for the
        /// virtual machine. This element is only used with the
        /// NetworkConfigurationSet type.
        /// </summary>
        public IList<InputEndpoint> InputEndpoints
        {
            get { return this._inputEndpoints; }
            set { this._inputEndpoints = value; }
        }
        
        private IList<NetworkInterface> _networkInterfaces;
        
        /// <summary>
        /// Optional.
        /// </summary>
        public IList<NetworkInterface> NetworkInterfaces
        {
            get { return this._networkInterfaces; }
            set { this._networkInterfaces = value; }
        }
        
        private string _networkSecurityGroup;
        
        /// <summary>
        /// Optional. Gets or sets the Network Security Group associated with
        /// this role. Optional
        /// </summary>
        public string NetworkSecurityGroup
        {
            get { return this._networkSecurityGroup; }
            set { this._networkSecurityGroup = value; }
        }
        
        private IList<ConfigurationSet.PublicIP> _publicIPs;
        
        /// <summary>
        /// Optional. Optional. A set of public IPs. Currently, only one
        /// additional public IP per role is supported in an IaaS deployment.
        /// The IP address is in addition to the default VIP for the
        /// deployment.
        /// </summary>
        public IList<ConfigurationSet.PublicIP> PublicIPs
        {
            get { return this._publicIPs; }
            set { this._publicIPs = value; }
        }
        
        private bool? _resetPasswordOnFirstLogon;
        
        /// <summary>
        /// Optional. Specifies whether password should be reset the first time
        /// the administrator logs in.
        /// </summary>
        public bool? ResetPasswordOnFirstLogon
        {
            get { return this._resetPasswordOnFirstLogon; }
            set { this._resetPasswordOnFirstLogon = value; }
        }
        
        private SshSettings _sshSettings;
        
        /// <summary>
        /// Optional. Specifies the SSH public keys and key pairs to populate
        /// in the image during provisioning. This element is only used with
        /// the LinuxProvisioningConfiguration set.
        /// </summary>
        public SshSettings SshSettings
        {
            get { return this._sshSettings; }
            set { this._sshSettings = value; }
        }
        
        private string _staticVirtualNetworkIPAddress;
        
        /// <summary>
        /// Optional. Specifies a Customer Address, i.e. an IP address assigned
        /// to a VM in a VNet's SubNet. For example: 10.0.0.4.
        /// </summary>
        public string StaticVirtualNetworkIPAddress
        {
            get { return this._staticVirtualNetworkIPAddress; }
            set { this._staticVirtualNetworkIPAddress = value; }
        }
        
        private IList<StoredCertificateSettings> _storedCertificateSettings;
        
        /// <summary>
        /// Optional. Contains a list of service certificates with which to
        /// provision to the new role. This element is only used with the
        /// WindowsProvisioningConfiguration set.
        /// </summary>
        public IList<StoredCertificateSettings> StoredCertificateSettings
        {
            get { return this._storedCertificateSettings; }
            set { this._storedCertificateSettings = value; }
        }
        
        private IList<string> _subnetNames;
        
        /// <summary>
        /// Optional. The list of Virtual Network subnet names that the
        /// deployment belongs to. This element is only used with the
        /// NetworkConfigurationSet type.
        /// </summary>
        public IList<string> SubnetNames
        {
            get { return this._subnetNames; }
            set { this._subnetNames = value; }
        }
        
        private string _timeZone;
        
        /// <summary>
        /// Optional. Specifies the time zone for the virtual machine. This
        /// element is only used with the WindowsProvisioningConfiguration
        /// set. For a complete list of supported time zone entries, you can
        /// refer to the values listed in the registry entry
        /// HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Windows
        /// NT\\CurrentVersion\\Time Zones on a computer running Windows 7,
        /// Windows Server 2008, and Windows Server 2008 R2 or you can use the
        /// tzutil command-line tool to list the valid time. The tzutil tool
        /// is installed by default on Windows 7, Windows Server 2008, and
        /// Windows Server 2008 R2.
        /// </summary>
        public string TimeZone
        {
            get { return this._timeZone; }
            set { this._timeZone = value; }
        }
        
        private string _userName;
        
        /// <summary>
        /// Optional. Specifies the name of a user to be created in the sudoer
        /// group of the virtual machine. User names are ASCII character
        /// strings 1 to 32 characters in length. This element is only used
        /// with the LinuxProvisioningConfiguration set.
        /// </summary>
        public string UserName
        {
            get { return this._userName; }
            set { this._userName = value; }
        }
        
        private string _userPassword;
        
        /// <summary>
        /// Optional. Specifies the password for user name. Passwords are ASCII
        /// character strings 6 to 72 characters in length. This element is
        /// only used with the LinuxProvisioningConfiguration set.
        /// </summary>
        public string UserPassword
        {
            get { return this._userPassword; }
            set { this._userPassword = value; }
        }
        
        private WindowsRemoteManagementSettings _windowsRemoteManagement;
        
        /// <summary>
        /// Optional. Configures the Windows Remote Management service on the
        /// virtual machine, which enables remote Windows PowerShell.
        /// </summary>
        public WindowsRemoteManagementSettings WindowsRemoteManagement
        {
            get { return this._windowsRemoteManagement; }
            set { this._windowsRemoteManagement = value; }
        }
        
        /// <summary>
        /// Initializes a new instance of the ConfigurationSet class.
        /// </summary>
        public ConfigurationSet()
        {
            this.InputEndpoints = new LazyList<InputEndpoint>();
            this.NetworkInterfaces = new LazyList<NetworkInterface>();
            this.PublicIPs = new LazyList<ConfigurationSet.PublicIP>();
            this.StoredCertificateSettings = new LazyList<StoredCertificateSettings>();
            this.SubnetNames = new LazyList<string>();
        }
        
        /// <summary>
        /// An additional public IP that will be created for the role. The
        /// public IP will be an additional IP for the role. The role
        /// continues to be addressable via the default deployment VIP.
        /// </summary>
        public partial class PublicIP
        {
            private string _domainNameLabel;
            
            /// <summary>
            /// Optional. The DNS name of the public IP.
            /// </summary>
            public string DomainNameLabel
            {
                get { return this._domainNameLabel; }
                set { this._domainNameLabel = value; }
            }
            
            private int? _idleTimeoutInMinutes;
            
            /// <summary>
            /// Optional. The idle timeout in minutes for this Public IP.
            /// </summary>
            public int? IdleTimeoutInMinutes
            {
                get { return this._idleTimeoutInMinutes; }
                set { this._idleTimeoutInMinutes = value; }
            }
            
            private string _name;
            
            /// <summary>
            /// Optional. The name of the public IP.
            /// </summary>
            public string Name
            {
                get { return this._name; }
                set { this._name = value; }
            }
            
            /// <summary>
            /// Initializes a new instance of the PublicIP class.
            /// </summary>
            public PublicIP()
            {
            }
        }
    }
}
